      PROGRAM    SPD_3D_TOSER_LAUNCH
      IMPLICIT   NONE 
      INCLUDE   'spd.i'
      CHARACTER    STR*32
      INTEGER*8    STACK_SIZE_IN_BYTES, GB, IS
      PARAMETER  ( GB = 1024*1024*1024 )
      PARAMETER  ( STACK_SIZE_IN_BYTES = SPD__STACK_SIZE_IN_GIGABYTES * GB )
      INTEGER*8, EXTERNAL :: SET_STACKSIZE 
!
! --- Set stacksize
!
      IS = SET_STACKSIZE ( %VAL(STACK_SIZE_IN_BYTES) )
      CALL INCH8    ( STACK_SIZE_IN_BYTES/INT8(1024), STR )
      CALL SETENV   ( 'GOMP_STACKSIZE'//CHAR(0), TRIM(STR)//CHAR(0), %VAL(1) )
      CALL SPD_3D_TOSER()
      END  PROGRAM  SPD_3D_TOSER_LAUNCH  !#!#
!
! ------------------------------------------------------------------------
!
      SUBROUTINE  SPD_3D_TOSER()
! ************************************************************************
! *                                                                      *
! *   Program SPD_3D_TOSER  reads the directory with ascii files with    *
! *   the 3D Slanted Path Delay, parses them, transforms to the binary   *
! *   format, and writes the output files in the binary spd format into  *
! *   the output directory.                                              *
! *                                                                      *
! *  ### 20-FEB-2009  SPD_3D_TOSER  v2.3 (c)  L. Petrov 03-NOV-2020 ###  *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'spd.i'
      INCLUDE   'spd_local.i'
      INCLUDE   'astro_constants.i'
      TYPE     ( SPD_3D__TYPE   ) :: SPD
      TYPE     ( SPD_DEL__TYPE  ) :: SPD_IN
      INTEGER*4  M_MOD, M_INP, M_FIL
      PARAMETER  ( M_MOD = 512 )
      PARAMETER  ( M_INP = 512 )
      PARAMETER  ( M_FIL = SPD__M_FIL )
      CHARACTER  DIRIN*128, DIRBSPD*128, FILBSPD(M_FIL)*128, FILIN(M_FIL)*128, &
     &           FILOUT*128, PREF_OUT*128, FIL_DAT*21, FILNAM*128, INP_FMT*128, &
     &           MOD_TEXT(M_MOD)*128, INP_TEXT(M_INP)*128, STR*128, MODE*8, POSTFIX*8, &
     &           CH_NL*1
      ADDRESS__TYPE :: DIR_DESC(16)
      REAL*8     TAI_END, FIL_SEC, TIM_STEP, EPS
      PARAMETER  ( EPS = 120.0 )
      INTEGER*4  L_MOD, L_INP, IND_STA, LEV, IS, J1, J2, J3, J4, J5, J6, &
     &           IL, L_FIL, L_BSPD, IVRB, ID, PID, NREC, MJD_END, FIL_MJD, IUER
#ifdef GNU
      INTEGER*4     NTHR, NTHR_SAVED
#else
      ADDRESS__TYPE NTHR, NTHR_SAVED
#endif
      LOGICAL*4  FL_OUT_TERM 
      CHARACTER, EXTERNAL :: MJDSEC_TO_DATE*30
      INTEGER*4, EXTERNAL :: I_LEN, ILEN, GETPID, GET_FILE_FROM_DIR, LINDEX, RENAME, &
     &                       SYSTEM
#ifdef GNU
      LOGICAL*4, INTRINSIC :: ISATTY
      INTRINSIC  FLUSH
      INTEGER*4, EXTERNAL     :: OMP_GET_MAX_THREADS, OMP_GET_NUM_THREADS 
#else
      LOGICAL*4, EXTERNAL  :: ISATTY
      ADDRESS__TYPE, EXTERNAL :: OMP_GET_MAX_THREADS, OMP_GET_NUM_THREADS 
#endif
!
      INP_FMT = 'SPD_3D_ASCII'
      FL_OUT_TERM = ISATTY ( 6 )
      IF ( FL_OUT_TERM ) THEN
           CH_NL = CHAR(13) ! New line character
         ELSE 
           CH_NL = CHAR(32) 
      END IF
!
      IVRB = 0
      IF ( IARGC() < 3 ) THEN
           IF ( IARGC() .GE. 1 ) THEN
                CALL GETARG ( 1, DIRIN )
                IF ( DIRIN == 'version'   .OR.  &
     &               DIRIN == '--version' .OR.  &
     &               DIRIN == '-v' ) THEN
!
                     WRITE ( 6, '(A)' ) SPD__VERSION
                     CALL EXIT ( 0 )
                 END IF
           END IF
           WRITE ( 6, '(A)' ) 'Usage: spd_3d_toser {dir_in} {pref_out} {create/update} [ivrb]'
           CALL EXIT ( 1 ) 
         ELSE 
           CALL GETARG ( 1, DIRIN )
           CALL GETARG ( 2, PREF_OUT )
           CALL GETARG ( 3, MODE     )
           IF ( IARGC() .GE. 4 ) THEN
                CALL GETARG ( 4, STR )
                CALL CHIN   ( STR, IVRB )
           END IF
      END IF
!
      IF ( MODE == 'create' .OR. MODE == 'update' .OR. MODE == 'summary' ) THEN
           CONTINUE 
         ELSE 
           IUER = -1
           CALL ERR_LOG ( 6501, IUER, 'SPD_3D_TOSER', 'Unsupported'// &
     &         '3d argument '//MODE(1:I_LEN(MODE))//' only '// &
     &         'create, update, or summary are supported' )
           CALL EXIT ( 1 )
      END IF
!
! --- Set the number of threads. We honor OMP_NUM_THREADS environment variable
!
      CALL GETENVAR ( 'OMP_NUM_THREADS', STR )
      IF ( ILEN(STR) == 0 ) THEN
           NTHR = 1
         ELSE 
           CALL CHIN ( STR, NTHR )
      END IF
      CALL OMP_SET_NUM_THREADS ( %VAL(NTHR) )
      NTHR = OMP_GET_MAX_THREADS()
!
! --- Initialize SPD object
!
      IUER = -1
      CALL SPD_INIT ( SPD, IUER )
!
      ID = LINDEX ( PREF_OUT, '/' )
      IF ( ID .LE. 1 ) THEN
           DIRBSPD = '.'
         ELSE 
           DIRBSPD = PREF_OUT(1:ID-1)
      END IF
!
      IF ( MODE == 'update' .OR. MODE == 'summary' ) THEN
!
! -------- Aga. We are in the update mode
!
           IUER = -1
           IF ( IVRB > 1 ) THEN
                WRITE ( 6, '(A)' ) '  Read BSPD files from directory '// &
     &                                DIRBSPD(1:I_LEN(DIRBSPD))
           END IF
!
! -------- Initialize SPD_IN object for input SPD delays
!
           CALL SPD_INIT ( SPD_IN, IUER )
!
! -------- Get information about PID of this process
!
           PID = GETPID()
           WRITE ( UNIT=POSTFIX(1:8), FMT='(I8)' ) PID
           CALL BLANK_TO_ZERO ( POSTFIX(1:8) )
!
! -------- Read the BSPD directory, extact relevant bspd files and read their headers
!
           LEV    = 0
           L_BSPD = 0
           NREC   = 0
           DO 410 J1=1,1024*1024
!
! ------------ Get the new file
!
               IS = GET_FILE_FROM_DIR ( LEV, DIR_DESC, DIRBSPD, FILNAM )
               IF ( LEV == 0 ) GOTO 810
               IF ( IS .NE. 0 ) THEN
                    IUER = -1
                    CALL ERR_LOG ( 6502, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                  'reading directory '//DIRBSPD(1:I_LEN(DIRBSPD))//' -- '// &
     &                   FILNAM )
                    CALL EXIT ( 1 )
               END IF
!
! ------------ check its name
!
               IF ( ILEN(FILNAM) < 5 ) GOTO 810
               IF ( FILNAM(ILEN(FILNAM)-4:ILEN(FILNAM)) .NE. '.bspd' ) GOTO 410
               IF ( INDEX ( FILNAM, '#' ) > 0 ) GOTO 410
!
! ------------ Put the file name in the list 
!
               L_BSPD = L_BSPD + 1
               FILBSPD(L_BSPD) = FILNAM
               IF ( IVRB > 0 ) THEN
                    WRITE ( 6, 110 ) J1, L_BSPD, FILBSPD(L_BSPD)(1:I_LEN(FILBSPD(L_BSPD))), CH_NL
                    IF ( .NOT. FL_OUT_TERM ) WRITE ( 6, * ) ' '
                    CALL FLUSH ( 6 )
 110                FORMAT ( '  Read BSPD file ', I6, ' ( ', I6, ' ) ', A, A$ )
               END IF
!
! ------------ Read the file header
!
               IUER = -1
               CALL SPD_3D_BIN_READ_HEAD ( FILNAM, SPD_IN, IUER )
               IF ( IUER .NE. 0 ) THEN
                    IUER = -1
                    CALL ERR_LOG ( 6503, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                  'reading the header of the input file '//FILNAM )
                    CALL EXIT ( 1 )
               END IF
!
               IF ( NREC == 0 ) THEN
!
! ----------------- This was the first file. Store some fields of the header
!
                    NREC     = SPD_IN%TIM%NREC
                    MJD_END  = SPD_IN%TIM%MJD_END
                    TAI_END  = SPD_IN%TIM%TAI_END
                    TIM_STEP = SPD_IN%TIM%TIM_STEP
                  ELSE 
!
! ----------------- This was not the first bspd file. Compare its header parameters
! ----------------- with the parameters from the 1st file. Any discreapancy is considered 
! ----------------- a fatal mistake
!
                    IF ( SPD_IN%TIM%NREC    .NE. NREC    .OR. &
     &                   SPD_IN%TIM%MJD_END .NE. MJD_END .OR. &
     &                   SPD_IN%TIM%TAI_END .NE. TAI_END      ) THEN
                         WRITE ( 6, * ) ' '
                         WRITE ( 6, * ) 'SPD_IN%TIM%NREC=    ', SPD_IN%TIM%NREC,    ' NREC=    ', NREC
                         WRITE ( 6, * ) 'SPD_IN%TIM%MJD_END= ', SPD_IN%TIM%MJD_END, ' MJD_END= ', MJD_END
                         WRITE ( 6, * ) 'SPD_IN%TIM%TAI_END= ', SPD_IN%TIM%TAI_END, ' TAI_END= ', TAI_END
                         IUER = -1
                         CALL ERR_LOG ( 6504, IUER, 'SPD_3D_TOSER', 'Find that '// &
     &                       'file '//FILBSPD(L_BSPD)(1:I_LEN(FILBSPD(L_BSPD)))// &
     &                       ' has different number of records and/or last epoch '// &
     &                       ' than '//FILBSPD(1) )
                         CALL EXIT ( 1 )
                      END IF
               END IF
 410       CONTINUE 
 810       CONTINUE 
           IF ( L_BSPD == 0 .OR. NREC == 0 ) THEN
!
! ------------- No relevant BSPD files were found? Set 'create' mode
!
                MODE = 'create'
           END IF
           IF ( L_BSPD > 0 ) THEN
!
! ------------- Sort bspd files
!
                CALL SORT_FAST_CH ( L_BSPD, FILBSPD )
           END IF
         ELSE 
           NREC    = 0
           MJD_END = 0
           TAI_END = 0.0D0
           TIM_STEP = 0.0D0
      END IF
      IF ( ( MODE == 'update' .OR. MODE == 'summary' ) .AND. IVRB > 1 ) THEN
           WRITE ( 6, * ) ' '
           IUER = -1
           STR = MJDSEC_TO_DATE ( MJD_END, TAI_END, IUER )
           WRITE ( 6, 120 ) STR(1:21), NREC
 120       FORMAT ( '  Last date of data in bspd files is ', A, &
     &              ' . They have ', I6 , ' records' )
      END IF
!
! --- Read directory DIRIN with SPD files
!
      IF ( IVRB > 0 ) THEN
           WRITE ( 6, '(A)' ) '  Read input directory '//DIRIN(1:I_LEN(DIRIN))
      END IF
      LEV = 0
      L_FIL = 0
      DO 420 J2=1,1024*1024*1024
         IS = GET_FILE_FROM_DIR ( LEV, DIR_DESC, DIRIN, FILNAM )
         IF ( LEV == 0 ) GOTO 820
         IF ( IS .NE. 0 ) THEN
              CALL ERR_LOG ( 6505, -2, 'SPD_3D_TOSER', 'Error in '// &
     &            'reading directory '//DIRIN(1:I_LEN(DIRIN))//' -- '// &
     &             FILNAM )
              CALL EXIT ( 1 )
         END IF
!
! ------ Check the file name
!
         IF ( ILEN(FILNAM) < 5 ) GOTO 820
         IF ( FILNAM(ILEN(FILNAM)-3:ILEN(FILNAM)) .NE. '.spd' ) GOTO 420
         IF ( INDEX ( FILNAM, '#' ) > 0 ) GOTO 420
!
         IF ( MODE == 'update' ) THEN
!
! ----------- Get the date embedded in the file name
!
              FIL_DAT = FILNAM(ILEN(FILNAM)-16:ILEN(FILNAM)-13)//'.'// &
     &                  FILNAM(ILEN(FILNAM)-12:ILEN(FILNAM)-11)//'.'// &
     &                  FILNAM(ILEN(FILNAM)-10:ILEN(FILNAM)-9)//'_'// &
     &                  FILNAM(ILEN(FILNAM)-7:ILEN(FILNAM)-6)//':'// &
     &                  FILNAM(ILEN(FILNAM)-5:ILEN(FILNAM)-4)//':00.0'
!
! ----------- Get the date from extracted from the file name
!
              IUER = -1
              CALL DATE_TO_TIME ( FIL_DAT, FIL_MJD, FIL_SEC, IUER )
              IF ( IUER .NE. 0 ) THEN
                   IUER = -1
                   CALL ERR_LOG ( 6506, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                 'reading directory '//DIRIN(1:I_LEN(DIRIN))//' -- '// &
     &                  FILNAM )
                   CALL EXIT ( 1 )
              END IF
              IF ( IVRB > 2 ) THEN
                   WRITE ( 6, * ) ' FIL_MJD = ', FIL_MJD, ' FIL_SEC= ', FIL_SEC
              END IF
              IF ( FIL_MJD*86400.0 + FIL_SEC - EPS < &
     &             MJD_END*86400.0 + TAI_END         ) THEN
!
! ---------------- Epoch of this file is too early. Skip it
!
                   GOTO 420
              END IF
         END IF
!
! ------ Add this file name to the list
!
         L_FIL = L_FIL + 1
         IF ( L_FIL > M_FIL ) THEN
              CALL CLRCH ( STR )
              CALL INCH  ( M_FIL, STR )
              CALL ERR_LOG ( 6507, -2, 'SPD_3D_TOSER', 'Too many spd files '// &
     &            'were found in '//DIRIN(1:I_LEN(DIRIN))//' directory: '// &
     &            'more than '//STR )
              CALL EXIT ( 1 )
         END IF
         FILIN(L_FIL) = FILNAM 
 420  CONTINUE 
 820  CONTINUE 
!
      IF ( L_BSPD > 0 ) THEN
           CALL SORT_FAST_CH ( L_BSPD, FILBSPD )
      END IF
      IF ( MODE == 'summary' ) THEN
           IF ( L_BSPD == 0 ) THEN
                IUER = -1
                CALL ERR_LOG ( 6508, IUER, 'SPD_3D_TOSER', 'No bdsp '// &
     &              'files was found in directory '//DIRBSPD )
                CALL EXIT ( 1 )
              ELSE
                IUER = -1
                CALL CREATE_BSPD_SUMMARY ( DIRBSPD, L_BSPD, FILBSPD, IUER )
                IF ( IUER .NE. 0 ) THEN
                     IUER = -1
                     CALL ERR_LOG ( 6509, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                   'an attempt to generate summary' )
                     CALL EXIT ( 1 )
                END IF
                CALL EXIT ( 0 ) 
           END IF
      END IF
!
      IF ( L_FIL == 0 ) THEN
           IF ( MODE == 'update' .AND. NREC > 0 ) THEN
                IUER = -1
                STR  = MJDSEC_TO_DATE ( MJD_END, TAI_END, IUER )
                IF ( IVRB > 1 ) WRITE ( 6, '(A)' ) ' '
                WRITE ( 6, '(A)' ) ' Nothing to update. The last date is '//STR(1:19)
!
                IUER = -1
                CALL CREATE_BSPD_SUMMARY ( DIRBSPD, L_BSPD, FILBSPD, IUER )
                IF ( IUER .NE. 0 ) THEN
                     IUER = -1
                     CALL ERR_LOG ( 6510, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                   'an attempt to create BSPD summary' )
                     CALL EXIT ( 1 )
                END IF
!
                CALL EXIT ( 0 )
              ELSE 
                CALL ERR_LOG ( 6511, -2, 'SPD_3D_TOSER', 'No spd files were '// &
     &              'found in '//DIRIN(1:I_LEN(DIRIN))//' directory' )
                CALL EXIT ( 1 )
           END IF
      END IF
      IF ( IVRB == 7 ) THEN
           WRITE ( 6, * ) 'Quit because IVRB=7'
           CALL EXIT ( 0 )
      END IF
!
! --- Sort files by name. This will force chronological sorting
!
      CALL SORT_FAST_CH ( L_FIL, FILIN )
      IF ( MODE == 'update' .AND. IVRB > 1 ) THEN
           STR = FILIN(1)(ILEN(FILIN(1))-16:ILEN(FILIN(1))-13)//'.'// &
     &                         FILIN(1)(ILEN(FILIN(1))-12:ILEN(FILIN(1))-11)//'.'// &
     &                         FILIN(1)(ILEN(FILIN(1))-10:ILEN(FILIN(1))-9)//'_'// &
     &                         FILIN(1)(ILEN(FILIN(1))-7:ILEN(FILIN(1))-6)//':'// &
     &                         FILIN(1)(ILEN(FILIN(1))-5:ILEN(FILIN(1))-4)//':00.0'
           FIL_DAT = FILIN(L_FIL)(ILEN(FILIN(L_FIL))-16:ILEN(FILIN(L_FIL))-13)//'.'// &
     &                  FILIN(L_FIL)(ILEN(FILIN(L_FIL))-12:ILEN(FILIN(L_FIL))-11)//'.'// &
     &                  FILIN(L_FIL)(ILEN(FILIN(L_FIL))-10:ILEN(FILIN(L_FIL))-9)//'_'// &
     &                  FILIN(L_FIL)(ILEN(FILIN(L_FIL))-7:ILEN(FILIN(L_FIL))-6)//':'// &
     &                  FILIN(L_FIL)(ILEN(FILIN(L_FIL))-5:ILEN(FILIN(L_FIL))-4)//':00.0'
           WRITE ( 6, 130 ) L_FIL, STR(1:21), FIL_DAT(1:21)
 130       FORMAT ( '  Found ', I5, ' new SPD files in a range ', A, 2X, A )
      END IF
!
      IF ( MODE == 'update' ) THEN
!
! -------- In the update mode we for safety make a temporary copy of bspd files.
! -------- We append a suffus with the PID to the file name
!
           IF ( IVRB > 0 ) THEN
                WRITE ( 6, 140 ) L_FIL, L_BSPD
                CALL FLUSH ( 6 )
 140            FORMAT ( '  Number of spd-files: ', I6, ' Number of bspd files: ', I6  )
           END IF
           DO 430 J3=1,L_BSPD
              IF ( IVRB > 0 ) THEN
                   WRITE ( 6, 150 ) J3, L_BSPD, FILBSPD(J3)(1:I_LEN(FILBSPD(J3))), CH_NL
                   IF ( .NOT. FL_OUT_TERM ) WRITE ( 6, * ) ' '
                   CALL FLUSH ( 6 )
 150               FORMAT ( '  Copy file ', I6, ' ( ', I6, ' ) ', A, A$ )
               END IF
               IS = SYSTEM ( 'cp '//FILBSPD(J3)(1:I_LEN(FILBSPD(J3)))//' '// &
     &                       FILBSPD(J3)(1:I_LEN(FILBSPD(J3)))//'__'//POSTFIX//CHAR(0) )
               IF ( IS .NE. 0 ) THEN
                    CALL CLRCH   ( STR )
                    CALL GERROR  ( STR )
                    CALL ERR_LOG ( 6512, -2, 'SPD_3D_TOSER', 'Error in '// &
     &                  'an attempt to copy file '// &
     &                  FILBSPD(J3)(1:I_LEN(FILBSPD(J3)))//' -- '//STR )
                    CALL EXIT ( 1 )
               END IF
               IS = SYSTEM ( 'chmod g+wr,o+wr '// &
     &                        FILBSPD(J3)(1:I_LEN(FILBSPD(J3)))//'__'//POSTFIX//CHAR(0) )
               FILBSPD(J3) = FILBSPD(J3)(1:I_LEN(FILBSPD(J3)))//'__'//POSTFIX
 430       CONTINUE 
 830       CONTINUE 
      END IF
!
      IF ( MODE == 'create' ) THEN
           L_BSPD = 0
      END IF
!
      DO 440 J4=1,L_FIL
         IF ( IVRB > 0 ) THEN
              WRITE ( 6, 160 ) J4, L_FIL, FILIN(J4)(1:I_LEN(FILIN(J4))), CH_NL
              IF ( .NOT. FL_OUT_TERM ) WRITE ( 6, * ) ' '
              CALL FLUSH ( 6 )
 160          FORMAT ( '  Process file ', I6, ' ( ', I6, ' ) ', A, A$ )
         END IF
!
! ------ Read the SPD ascii file
!
         IUER = -1
         CALL SPD_3D_READ ( FILIN(J4), INP_FMT, SPD, M_MOD, L_MOD, &
     &                      MOD_TEXT, M_INP, L_INP, INP_TEXT, IUER )
         IF ( IUER .NE. 0 ) THEN
              IUER = -1
              CALL ERR_LOG ( 6513, IUER, 'SPD_3D_TOSER', 'Error in reading '// &
     &            'the input SPD file '//FILIN(J4) )
              CALL EXIT ( 1 )
         END IF
!
! ------ Now we run the cycle over stations
!
!$OMP    PARALLEL DO IF ( NTHR > 1 ), &
!$OMP    PRIVATE ( J5, IUER, IL, FILOUT )
         DO 450 J5=1,SPD%NSTA
!
! --------- Build the name of the output bspd file
!
            CALL CLRCH ( FILOUT )
            FILOUT = PREF_OUT
            IL = ILEN(FILOUT)
            CALL TRAN ( 12, SPD%STA(J5)%NAME, FILOUT(ILEN(FILOUT)+1:) )
            FILOUT = FILOUT(1:IL+8)//'.bspd'
            CALL BLANK_TO_UNDSCR ( FILOUT(1:I_LEN(FILOUT)) )
!
            IF ( MODE == 'update' ) THEN
!
! -------------- Do not forget the suffix in the update mode, since we will update
! -------------- the saved copy
!
                 FILOUT = FILOUT(1:I_LEN(FILOUT))//'__'//POSTFIX
            END IF
            IF ( MODE == 'create' .AND. J4 == 1 ) THEN
!
! -------------- Create the output file for the j5-th file with contents of the J4-th epoch
!
                 IUER = -1
                 CALL SPD_3D_WRITE ( SPD__WRITE_BIN, J5, SPD, FILOUT, &
     &                               L_MOD, MOD_TEXT, L_INP, INP_TEXT, 3, IUER )
                 IF ( IUER .NE. 0 ) THEN
!$OMP                 CRITICAL
                      WRITE ( 6, * ) ' '
                      IUER = -1
                      CALL ERR_LOG ( 6514, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                    'writing the output file '//FILOUT )
                      CALL EXIT ( 1 )
!$OMP                 END CRITICAL
                 END IF
                 IS = SYSTEM ( 'chmod g+wr,o+wr '//FILOUT(1:I_LEN(FILOUT))//CHAR(0) )
!
!$OMP            CRITICAL
                 L_BSPD = L_BSPD + 1
!$OMP            END CRITICAL
                 FILBSPD(L_BSPD) = FILOUT
              ELSE
!
! -------------- Update the output file with the contents of the J4th epoch
!
                 IUER = -1
                 CALL SPD_3D_WRITE_DEL_ADD ( J4+NREC, J5, SPD, FILOUT, IUER )
                 IF ( IUER .NE. 0 ) THEN
!$OMP                 CRITICAL
                      WRITE ( 6, * ) ' '
                      IUER = -1
                      CALL ERR_LOG ( 6515, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &                    'writing the output file '//FILOUT )
                      CALL EXIT ( 1 )
!$OMP                 END CRITICAL
                 END IF
            END IF
 450     CONTINUE 
!$OMP    END PARALLEL DO
         CALL SPD_FREE ( SPD, 2 ) 
 440  CONTINUE 
      IF ( IVRB > 0 ) THEN
           WRITE ( 6, '(A)' ) ' '
      END IF
      IF ( MODE == 'update' ) THEN
!
! -------- Rename files back with locking
!
#ifndef SPC
           IF ( IVRB > 0 ) THEN
                WRITE ( 6, '(A)' ) 'Renaming temporary files'
           END IF
           IUER = -1
           CALL BSPD_MOVE_LOCK ( L_BSPD, FILBSPD, '__'//POSTFIX, IUER )
           IF ( IUER .NE. 0 ) THEN
                IUER = -1
                CALL ERR_LOG ( 6516, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &              'an attempt to rename file with postfix '// &
     &              '__'//POSTFIX )
                CALL EXIT ( 1 )
           END IF
#else
           IF ( IVRB > 0 ) THEN
                WRITE ( 6, '(A)' ) 'Renaming temporary files is DISABLED'
           END IF
#endif 
      END IF
!
      IUER = -1
!      IF ( MODE == 'update' ) THEN
!           CALL CREATE_BSPD_SUMMARY ( DIRBSPD, L_BSPD, FILBSPD, IUER )
!         ELSE IF ( MODE == 'create' ) THEN
!           CALL CREATE_BSPD_SUMMARY ( DIRBSPD, L_FIL,  FILIN,   IUER )
!      END IF
!
      CALL CREATE_BSPD_SUMMARY ( DIRBSPD, L_BSPD, FILBSPD, IUER )
      IF ( IUER .NE. 0 ) THEN
           IUER = -1
           CALL ERR_LOG ( 6517, IUER, 'SPD_3D_TOSER', 'Error in '// &
     &         'an attempt to create BSPD summary' )
           CALL EXIT ( 1 )
      END IF
!
! --- Hurrah!
!
      CALL EXIT ( 0 )
      END  SUBROUTINE  SPD_3D_TOSER  !#!#
!
! ------------------------------------------------------------------------
!
      SUBROUTINE BLANK_TO_UNDSCR ( STR )
      IMPLICIT   NONE 
      CHARACTER  STR*(*)
      INTEGER*4  J1
      DO 410 J1=1,LEN(STR)
         IF ( STR(J1:J1) == ' ' ) STR(J1:J1) = '_'
 410  CONTINUE 
      RETURN
      END  SUBROUTINE  BLANK_TO_UNDSCR  !#!#
!
! ------------------------------------------------------------------------
!
      SUBROUTINE CREATE_BSPD_SUMMARY ( DIROUT, L_BSPD, FILBSPD, IUER )
! ************************************************************************
! *                                                                      *
! *   Routine CREATE_BSPD_SUMMARY 
! *                                                                      *
! * ## 24-JUN-2014 CREATE_BSPD_SUMMARY v1.1 (c) L. Petrov 15-SEP-2014 ## *
! *                                                                      *
! ************************************************************************
      IMPLICIT   NONE 
      INCLUDE   'spd.i'
      INCLUDE   'spd_local.i'
      INCLUDE   'astro_constants.i'
      TYPE     ( SPD_DEL__TYPE  ) :: SPD_DEL
      INTEGER*4  L_BSPD, IUER
      CHARACTER  DIROUT*(*), FILBSPD(L_BSPD)*(*)
      TYPE     ( SPD_DEL__TYPE  ) :: SPD_IN
      CHARACTER  BUF(SPD__M_STA+16)*128, STR_BEG*21, STR_END*21, FILOUT*128, &
     &           STR*128, LONG_STR*32768
      INTEGER*4  J1, J2, J3, NF, IP, IN, N1, IER
      CHARACTER, EXTERNAL :: GET_CDATE*19, MJDSEC_TO_DATE*30
      INTEGER*4, EXTERNAL :: ILEN, I_LEN  
!
      NF = 0
      DO 410 J1=1,L_BSPD
         CALL ERR_PASS ( IUER, IER )
         CALL SPD_3D_BIN_READ_HEAD ( FILBSPD(J1), SPD_DEL, IER )
         IF ( IER .NE. 0 ) THEN
              CALL ERR_LOG ( 6531, IUER, 'CREATE_BSPD_SUMMARY', 'Error '// &
     &            'in reading the input SPD file '//FILBSPD(J1) )
              RETURN 
         END IF
         IF ( J1 == 1 ) THEN
              NF = NF + 1; BUF(NF) = BSPD_SUMM__LABEL  
              NF = NF + 1; BUF(NF) = '#'
              NF = NF + 1; BUF(NF) = 'Last_Update:        '//GET_CDATE()
              STR_BEG = MJDSEC_TO_DATE ( SPD_DEL%TIM%MJD_BEG, SPD_DEL%TIM%TAI_BEG, IER )
              STR_END = MJDSEC_TO_DATE ( SPD_DEL%TIM%MJD_END, SPD_DEL%TIM%TAI_END, IER )
              NF = NF + 1
              WRITE ( UNIT=BUF(NF), FMT='("Min_epoch: ",9X,I5,1X,F7.1, 1X, A)' ) &
     &                SPD_DEL%TIM%MJD_BEG, SPD_DEL%TIM%TAI_BEG, STR_BEG(1:21)
              NF = NF + 1
              WRITE ( UNIT=BUF(NF), FMT='("Max_epoch: ",9X,I5,1X,F7.1, 1X, A)' ) &
     &                SPD_DEL%TIM%MJD_END, SPD_DEL%TIM%TAI_END, STR_END(1:21)
!
              NF = NF + 1
              WRITE ( UNIT=BUF(NF), FMT='("Num_Epoch: ",5X,I9)' ) &
     &                SPD_DEL%TIM%NREC
!
              NF = NF + 1
              WRITE ( UNIT=BUF(NF), FMT='("Num_Stations: ",2X,I9)' ) L_BSPD
!
              NF = NF + 1
              WRITE ( UNIT=BUF(NF), FMT='("Sample_Interval: ",F8.1, " sec,  ", F8.6, " days")' ) &
     &                SPD_DEL%TIM%TIM_STEP, SPD_DEL%TIM%TIM_STEP/86400.0D0
!
              NF = NF + 1
              IF ( SPD_DEL%MOD%N_RFR .LE. 1 ) THEN
                   WRITE ( UNIT=BUF(NF), FMT='("Num_Refr:     ",10X,I1,2X,A)' ) &
     &                     SPD_DEL%MOD%N_RFR, SPD_DEL%MOD%SPD_TYPE(1)
                 ELSE IF ( SPD_DEL%MOD%N_RFR .GE. 2 ) THEN
                   WRITE ( UNIT=BUF(NF), FMT='("Num_Refr:     ",10X,I1,2X,A,2X,A)' ) &
     &                     SPD_DEL%MOD%N_RFR, SPD_DEL%MOD%SPD_TYPE(1), SPD_DEL%MOD%SPD_TYPE(2)
              END IF
              NF = NF + 1; BUF(NF) = '#'
              NF = NF + 1; BUF(NF) = '#  Description of the algorithm:'
              NF = NF + 1; BUF(NF) = '#'
!
              CALL LIB$MOVC3 ( SPD_DEL%MOD%LEN_TEXT, %REF(SPD_DEL%MOD%TEXT), %REF(LONG_STR) )
              IP = 1
              DO 420 J2=1,SPD_DEL%MOD%N_LINES
                 IN = INDEX ( LONG_STR(IP:SPD_DEL%MOD%LEN_TEXT), CHAR(0) ) + IP-1
                 NF = NF + 1
                 IF ( IN > IP ) THEN
                      WRITE ( BUF(NF), '(A)' ) '@@  '//LONG_STR(IP:IN-1)
                    ELSE 
                      WRITE ( BUF(NF), '(A)' ) '@@  '
                 END IF
                 IP = IN + 1
 420          CONTINUE 
              NF = NF + 1; BUF(NF) = '#'
              NF = NF + 1; BUF(NF) = '#  Description of numerical weather model:'
              NF = NF + 1; BUF(NF) = '#'
!
              CALL LIB$MOVC3 ( SPD_DEL%MET%LEN_TEXT, %REF(SPD_DEL%MET%TEXT), %REF(LONG_STR) )
              IP = 1
              DO 430 J3=1,SPD_DEL%MET%N_LINES
                 IN = INDEX ( LONG_STR(IP:SPD_DEL%MET%LEN_TEXT), CHAR(0) ) + IP-1
                 NF = NF + 1
                 IF ( IN > IP ) THEN
                      WRITE ( BUF(NF), '(A)' ) '%%  '//LONG_STR(IP:IN-1)
                    ELSE 
                      WRITE ( BUF(NF), '(A)' ) '%%  '
                 END IF
                 IP = IN + 1
 430          CONTINUE 
!
              NF = NF + 1; BUF(NF) = '#'
              N1 = NF + 1
         END IF
!
         NF = NF + 1
         WRITE ( UNIT=BUF(NF), FMT=160 ) &
     &                         SPD_DEL%STA%NAME, &
     &                         SPD_DEL%STA%PHI_GDT/DEG__TO__RAD, &
     &                         SPD_DEL%STA%LON/DEG__TO__RAD, &
     &                         SPD_DEL%STA%HEI_ELL, &
     &                         SPD_DEL%STA%HEI_GEOID
 160     FORMAT ( 'Station_name: ',A, ' Lat_gdt: ', F9.5, ' Lon: ', F9.5, &
     &           ' Hei_ell: ', F8.2, ' Hei_geo: ', F8.2 )
 410  CONTINUE 
!
      CALL SORT_FAST_CH ( NF-N1+1, BUF(N1) )
!
      FILOUT = DIROUT(1:I_LEN(DIROUT))//'/bspd_summary.txt'
      CALL ERR_PASS ( IUER, IER )
      CALL WR_TEXT  ( NF, BUF, FILOUT, IER )
      IF ( IER .NE. 0 ) THEN
           CALL ERR_LOG ( 6532, IUER, 'CREATE_BSPD_SUMMARY', 'Error '// &
     &         'in writing into the output file '//FILOUT )
           RETURN
      END IF
!
      CALL ERR_LOG ( 0, IUER )
      RETURN
      END  SUBROUTINE  CREATE_BSPD_SUMMARY  !#!  
